@@ -348,7 +348,7 @@ set_left_handed (GsdMouseManager *manager,
                 return;
 
         xdevice = open_gdk_device (device);
-        if (!xdevice)
+        if (xdevice == NULL)
                 return;
 
         /* If the device is a touchpad, swap tap buttons
@@ -447,6 +447,8 @@ set_motion (GsdMouseManager *manager,
 
         /* Get the list of feedbacks for the device */
         states = XGetFeedbackControl (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice, &num_feedbacks);
+        if (states == NULL)
+                num_feedbacks = 0;
         state = (XFeedbackState *) states;
         for (i = 0; i < num_feedbacks; i++) {
                 if (state->class == PtrFeedbackClass) {
@@ -485,6 +487,7 @@ set_middle_button (GsdMouseManager *manager,
         int format;
         unsigned long nitems, bytes_after;
         unsigned char *data;
+        int rc;
 
         prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                             "Evdev Middle Button Emulation", True);
@@ -498,26 +501,23 @@ set_middle_button (GsdMouseManager *manager,
 
         gdk_error_trap_push ();
 
-        XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                             xdevice, prop, 0, 1, False, XA_INTEGER, &type, &format,
                             &nitems, &bytes_after, &data);
 
-        if ((gdk_error_trap_pop () != 0)) {
-                XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-                return;
-        }
-
-        if (format == 8 && type == XA_INTEGER && nitems == 1) {
+        if (rc == Success && format == 8 && type == XA_INTEGER && nitems == 1) {
                 data[0] = middle_button ? 1 : 0;
 
-                gdk_error_trap_push ();
                 XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                        xdevice, prop, type, format, PropModeReplace, data, nitems);
-
-                gdk_error_trap_pop_ignored ();
         }
 
-        XFree (data);
+        if (gdk_error_trap_pop ())
+                g_warning ("Error in setting middle button emulation on \"%s\"", gdk_device_get_name (device));
+
+        if (rc == Success)
+                XFree (data);
+
         XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
@@ -598,10 +598,11 @@ set_tap_to_click (GdkDevice *device,
 
         if (rc == Success)
                 XFree (data);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-        if (gdk_error_trap_pop ()) {
+
+        if (gdk_error_trap_pop ())
                 g_warning ("Error in setting tap to click on \"%s\"", gdk_device_get_name (device));
-        }
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
 static void
@@ -655,11 +656,14 @@ set_horiz_scroll (GdkDevice *device,
                                        PropModeReplace, data, nitems);
         }
 
-        XFree (data);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-
         if (gdk_error_trap_pop ())
                 g_warning ("Error in setting horiz scroll on \"%s\"", gdk_device_get_name (device));
+
+        if (rc == Success)
+                XFree (data);
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
+
 }
 
 static void
@@ -713,11 +717,13 @@ set_edge_scroll (GdkDevice               *device,
                                        PropModeReplace, data, nitems);
         }
 
-        XFree (data);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-
         if (gdk_error_trap_pop ())
                 g_warning ("Error in setting edge scroll on \"%s\"", gdk_device_get_name (device));
+
+        if (rc == Success)
+                XFree (data);
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
 static void
@@ -744,14 +750,11 @@ set_touchpad_enabled (GdkDevice *device,
         XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice,
                                prop_enabled, XA_INTEGER, 8,
                                PropModeReplace, &data, 1);
-        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
-        gdk_flush ();
 
-        if (gdk_error_trap_pop ()) {
-                g_warning ("Error %s device \"%s\"",
-                           (state) ? "enabling" : "disabling",
-                           gdk_device_get_name (device));
-        }
+        if (gdk_error_trap_pop ())
+                g_warning ("Error %s device \"%s\"", (state) ? "enabling" : "disabling", gdk_device_get_name (device));
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
 }
 
 static void
@@ -976,8 +979,14 @@ device_removed_cb (GdkDeviceManager *device_manager,
                    GsdMouseManager  *manager)
 {
         if (gdk_device_get_source (device) == GDK_SOURCE_MOUSE) {
+                int id;
+
                 run_custom_command (device, COMMAND_DEVICE_REMOVED);
 
+                g_object_get (G_OBJECT (device), "device-id", &id, NULL);
+                g_hash_table_remove (manager->priv->blacklist,
+                                     GINT_TO_POINTER (id));
+
                 /* If a touchpad was to disappear... */
                 set_disable_w_typing (manager, g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TOUCHPAD_DISABLE_W_TYPING));
         }
